@@ -40,9 +40,12 @@ GType anjuta_session_phase_get_type (void) G_GNUC_CONST;
 /* enumerations from "anjuta-serializer.h" */
 GType anjuta_serializer_mode_get_type (void) G_GNUC_CONST;
 #define ANJUTA_TYPE_SERIALIZER_MODE (anjuta_serializer_mode_get_type())
-/* enumerations from "anjuta-vcs-status-tree-view.h" */
+/* enumerations from "anjuta-vcs-status.h" */
 GType anjuta_vcs_status_get_type (void) G_GNUC_CONST;
 #define ANJUTA_TYPE_VCS_STATUS (anjuta_vcs_status_get_type())
+/* enumerations from "gbf-project.h" */
+GType if_if_get_type (void) G_GNUC_CONST;
+#define ANJUTA_TYPE_IF (if_if_get_type())
 G_END_DECLS
 
 #endif /* __ANJUTA_ENUM_TYPES_H__ */
