@@ -52,6 +52,9 @@
 #endif
 
 #define ICON_FILE "anjuta-build-basic-autotools-plugin-48.png"
+#define ANJUTA_PIXMAP_BUILD            "anjuta-build"
+#define ANJUTA_STOCK_BUILD             "anjuta-build"
+
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-build-basic-autotools-plugin.ui"
 #define MAX_BUILD_PANES 3
 #define PREF_INDICATORS_AUTOMATIC "indicators.automatic"
@@ -136,6 +139,18 @@ typedef struct
 	gint file_saved;
 } BuildContext;
 
+/* Build function type */
+typedef BuildContext* (*BuildFunc) (BasicAutotoolsPlugin *plugin, const gchar *name,
+                                    IAnjutaBuilderCallback callback, gpointer user_data,
+                                    GError **err);
+
+typedef struct
+{
+	gchar *args;
+	gchar *name;
+	BuildFunc func;
+} BuildConfigureAndBuild;
+
 /* Declarations */
 static void update_project_ui (BasicAutotoolsPlugin *bb_plugin);
 static void update_configuration_menu (BasicAutotoolsPlugin *plugin);
@@ -171,6 +186,101 @@ static void on_root_check_toggled(GtkWidget* toggle_button, GtkWidget* entry)
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle_button)));
 }
 
+static gboolean
+project_has_configure (BasicAutotoolsPlugin *bb_plugin)
+{
+	gchar *configure;
+
+	if (bb_plugin->project_root_dir == NULL) return FALSE;
+	
+	configure = g_build_filename (bb_plugin->project_root_dir, "configure.ac", NULL);
+	if (!g_file_test (configure, G_FILE_TEST_EXISTS))
+	{
+		g_free (configure);
+		configure = g_build_filename (bb_plugin->project_root_dir, "configure.in", NULL);
+		if (!g_file_test (configure, G_FILE_TEST_EXISTS))
+		{
+			g_free (configure);
+
+			return FALSE;
+		}
+	}
+	g_free (configure);
+
+	return TRUE;
+}
+
+static gboolean
+directory_has_makefile (const gchar *dirname)
+{
+	gchar *makefile;
+	gboolean makefile_exists;
+	
+	makefile_exists = TRUE;	
+	makefile = g_build_filename (dirname, "Makefile", NULL);
+	if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
+	{
+		g_free (makefile);
+		makefile = g_build_filename (dirname, "makefile", NULL);
+		if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
+		{
+			g_free (makefile);
+			makefile = g_build_filename (dirname, "MAKEFILE", NULL);
+			if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
+			{
+				makefile_exists = FALSE;
+			}
+		}
+	}
+	g_free (makefile);
+	return makefile_exists;
+}
+
+static gboolean
+directory_has_file (const gchar *dirname, const gchar *filename)
+{
+	gchar *filepath;
+	gboolean exists;
+	
+	exists = TRUE;	
+	filepath = g_build_filename (dirname, filename, NULL);
+	if (!g_file_test (filepath, G_FILE_TEST_EXISTS))
+		exists = FALSE;
+	
+	g_free (filepath);
+	return exists;
+}
+
+static gchar*
+escape_label (const gchar *str)
+{
+	GString *ret;
+	const gchar *iter;
+	
+	ret = g_string_new ("");
+	iter = str;
+	while (*iter != '\0')
+	{
+		if (*iter == '_')
+		{
+			ret = g_string_append (ret, "__");
+			iter++;
+		}
+		else
+		{
+			const gchar *start;
+			const gchar *end;
+			
+			start = iter;
+			iter = g_utf8_next_char (iter);
+			end = iter;
+			
+			ret = g_string_append_len (ret, start, end - start);
+		}
+	}
+	return g_string_free (ret, FALSE);
+}
+
 /* Indicator locations reported by the build */
 
 static BuildIndicatorLocation*
@@ -271,7 +381,7 @@ build_context_pop_dir (BuildContext *context, const gchar *key,
 	
 	if (strcmp (top_dir, dir) != 0)
 	{
-		DEBUG_PRINT("Directory stack misaligned!!");
+		DEBUG_PRINT("%s", "Directory stack misaligned!!");
 	}
 	g_free (top_dir);
 	if (dir_stack)
@@ -619,8 +729,8 @@ parse_error_line (const gchar * line, gchar ** filename, int *lineno)
 	}
 
       down:
-	i = strlen (line) - 1;
-	while (isspace (line[i]) == FALSE)
+	i = strlen (line);
+	do
 	{
 		i--;
 		if (i < 0)
@@ -629,7 +739,7 @@ parse_error_line (const gchar * line, gchar ** filename, int *lineno)
 			*lineno = 0;
 			return FALSE;
 		}
-	}
+	} while (isspace (line[i]) == FALSE);
 	k = i++;
 	while (line[i++] != ':')
 	{
@@ -778,11 +888,17 @@ on_build_mesg_format (IAnjutaMessageView *view, const gchar *one_line,
 			type = IANJUTA_MESSAGE_VIEW_TYPE_WARNING;
 			indicator = IANJUTA_INDICABLE_WARNING;
 		}
-		else
+		else if ((strstr (line, "error:") != NULL) ||
+		          (strstr (line, _("error:")) != NULL))
 		{
 			type = IANJUTA_MESSAGE_VIEW_TYPE_ERROR;
 			indicator = IANJUTA_INDICABLE_CRITICAL;
 		}
+		else
+		{
+			type = IANJUTA_MESSAGE_VIEW_TYPE_NORMAL;
+			indicator = IANJUTA_INDICABLE_IMPORTANT;
+		}
 		
 		mid_str = strstr (line, dummy_fn);
 		DEBUG_PRINT ("mid_str = %s, line = %s", mid_str, line);
@@ -939,12 +1055,12 @@ on_build_terminated (AnjutaLauncher *launcher,
 		if (status)
 		{
 			ianjuta_message_view_buffer_append (context->message_view,
-									_("Completed unsuccessful\n"), NULL);
+									_("Completed unsuccessfully\n"), NULL);
 		}
 		else
 		{
 			ianjuta_message_view_buffer_append (context->message_view,
-									   _("Completed successful\n"), NULL);
+									   _("Completed successfully\n"), NULL);
 		}
 		ianjuta_message_view_buffer_append (context->message_view, buff1, NULL);
 		g_free (buff1);
@@ -963,7 +1079,7 @@ on_build_terminated (AnjutaLauncher *launcher,
 static void
 on_message_view_destroyed (BuildContext *context, GtkWidget *view)
 {
-	DEBUG_PRINT ("Destroying build context");
+	DEBUG_PRINT ("%s", "Destroying build context");
 	context->message_view = NULL;
 	
 	build_context_destroy_view (context);
@@ -1342,7 +1458,7 @@ build_build_file_or_dir (BasicAutotoolsPlugin *plugin, const gchar *name,
 										   "%s %s",
 										   CHOOSE_COMMAND (plugin, BUILD),
 										   target ? target : "");
-	if (callback) build_program_set_callback (prog, callback, user_data);
+	build_program_set_callback (prog, callback, user_data);
 	
 	context = build_save_and_execute_command (plugin, prog, TRUE, err);
 	g_free (target);
@@ -1360,7 +1476,6 @@ build_is_file_built (BasicAutotoolsPlugin *plugin, const gchar *filename,
 {
 	BuildContext *context;
 	gchar *build_dir;
-	gchar *src_dir;
 	gchar *target;
 	BuildProgram *prog;
 	
@@ -1404,18 +1519,22 @@ get_root_install_command(BasicAutotoolsPlugin *bplugin)
 
 static BuildContext*
 build_install_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname,
+                   IAnjutaBuilderCallback callback, gpointer user_data,
 				   GError **err)
 {
 	BuildContext *context;
 	gchar* root = get_root_install_command(plugin);
 	gchar *build_dir = build_dir_from_source (plugin, dirname);
+	BuildProgram *prog;
 
-	context = build_save_and_execute_command (plugin,
-									 build_program_new_with_command (build_dir,
-																	 "%s %s",
-																	 root,
-																	 CHOOSE_COMMAND (plugin, INSTALL)),
-									 TRUE, err);
+	prog = build_program_new_with_command (build_dir,
+	                                       "%s %s",
+	                                       root,
+	                                       CHOOSE_COMMAND (plugin, INSTALL)),
+	build_program_set_callback (prog, callback, user_data);	
+	
+	context = build_save_and_execute_command (plugin, prog, TRUE, err);
+	
 	g_free (build_dir);
 	g_free(root);
 	
@@ -1574,31 +1693,47 @@ build_project_configured (GObject *sender,
 							GError *error,
 							gpointer user_data)
 {
-	if (error != NULL) return;
+	BuildConfigureAndBuild *pack = (BuildConfigureAndBuild *)user_data;
+	
+	if (error == NULL)
+	{
+		BuildContext *context = (BuildContext *)handle;
+		BasicAutotoolsPlugin *plugin = (BasicAutotoolsPlugin *)(context == NULL ? (void *)sender : (void *)context->plugin);
+		GValue *value;
+		gchar *uri;
+	
+		/* FIXME: check if build directory correspond, configuration could have changed */
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+	
+		uri = build_configuration_list_get_build_uri (plugin->configurations, build_configuration_list_get_selected (plugin->configurations));
+		g_value_set_string (value, uri);
+		g_free (uri);
+	
+		anjuta_shell_add_value (ANJUTA_PLUGIN (plugin)->shell, IANJUTA_BUILDER_ROOT_URI, value, NULL);
+	
+		update_configuration_menu (plugin);
 
-	BuildContext *context = (BuildContext *)handle;
-	BasicAutotoolsPlugin *plugin = (BasicAutotoolsPlugin *)(context == NULL ? (void *)sender : (void *)context->plugin);
-	GValue *value;
-	gchar *uri;
-	
-	/* FIXME: check if build directory correspond, configuration could have changed */
-	value = g_new0 (GValue, 1);
-	g_value_init (value, G_TYPE_STRING);
-	
-	uri = build_configuration_list_get_build_uri (plugin->configurations, build_configuration_list_get_selected (plugin->configurations));
-	g_value_set_string (value, uri);
-	g_free (uri);
-	
-	anjuta_shell_add_value (ANJUTA_PLUGIN (plugin)->shell, IANJUTA_BUILDER_ROOT_URI, value, NULL);
-	
-	update_configuration_menu (plugin);
+		/* Call build function if necessary */
+		if ((pack) && (pack->func != NULL)) pack->func (plugin, pack->name, NULL, NULL, NULL);
+	}
+
+	if (pack)
+	{
+		g_free (pack->args);
+		g_free (pack->name);
+		g_free (pack);
+	}
 }
 
 static BuildContext*
-build_configure_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gchar *args)
+build_configure_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gchar *args,
+                     BuildFunc func, const gchar *name)
 {
 	BuildContext *context;
 	BuildProgram *prog;
+	BuildConfigureAndBuild *pack = g_new(BuildConfigureAndBuild, 1);
+
 	
 	prog = build_program_new_with_command (dirname,
 										   "%s%s%s %s",
@@ -1606,7 +1741,10 @@ build_configure_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const g
 										   G_DIR_SEPARATOR_S,
 										   CHOOSE_COMMAND (plugin, CONFIGURE),
 										   args);
-	build_program_set_callback (prog, build_project_configured, NULL);
+	pack->args = NULL;
+	pack->func = func;
+	pack->name = g_strdup (name);
+	build_program_set_callback (prog, build_project_configured, pack);
 	
 	context = build_save_and_execute_command (plugin, prog, TRUE, NULL);
 	
@@ -1621,23 +1759,29 @@ build_configure_after_autogen (GObject *sender,
 							   GError *error,
 							   gpointer user_data)
 {
+	BuildConfigureAndBuild *pack = (BuildConfigureAndBuild *)user_data;
+	
 	if (error == NULL)
 	{
 		BuildContext *context = (BuildContext *)handle;
 		BasicAutotoolsPlugin *plugin = (BasicAutotoolsPlugin *)context->plugin;
 		struct stat conf_stat, log_stat;
 		gchar *filename;
+		gboolean has_configure;
 		
 		filename = g_build_filename (plugin->project_root_dir, "configure", NULL);
-		if (stat (filename, &conf_stat) != 0)
+		has_configure = stat (filename, &conf_stat) == 0;
+		g_free (filename);
+			
+		if (has_configure)
 		{
-			anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell), _("Can not configure project: Missing configure script in %s."), plugin->project_root_dir);
-		}
-		else
-		{
-			g_free (filename);
+			gboolean older;
+			
 			filename = g_build_filename (context->program->work_dir, "config.log", NULL);
-			if ((stat (filename, &log_stat) != 0) || (log_stat.st_mtime < conf_stat.st_mtime))
+			older =(stat (filename, &log_stat) != 0) || (log_stat.st_mtime < conf_stat.st_mtime);
+			g_free (filename);
+			
+			if (older)
 			{
 				/* configure has not be run, run it */
 				BuildProgram *prog;
@@ -1647,23 +1791,38 @@ build_configure_after_autogen (GObject *sender,
 													   plugin->project_root_dir,
 													   G_DIR_SEPARATOR_S,
 													   CHOOSE_COMMAND (plugin, CONFIGURE),
-													   (gchar *)user_data);
-				build_program_set_callback (prog, build_project_configured, NULL);
+													   pack != NULL ? pack->args : NULL);
+				build_program_set_callback (prog, build_project_configured, pack);
 		
 				build_set_command_in_context (context, prog);
 				build_execute_command_in_context (context, NULL);
 			}
+			else
+			{
+				/* run next command if needed */
+				build_project_configured (sender, handle, NULL, pack);
+			}
+			
+			return;
 		}
-		g_free (filename);
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell), _("Can not configure project: Missing configure script in %s."), plugin->project_root_dir);
+	}
+
+	if (pack)
+	{
+		g_free (pack->args);
+		g_free (pack->name);
+		g_free (pack);
 	}
-	g_free (user_data);
 }
 
 static BuildContext*
-build_generate_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gchar *args)
+build_generate_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gchar *args,
+                    BuildFunc func, const gchar *name)
 {
 	BuildContext *context;
 	BuildProgram *prog;
+	BuildConfigureAndBuild *pack = g_new (BuildConfigureAndBuild, 1);
 	
 	if (directory_has_file (plugin->project_root_dir, "autogen.sh"))
 	{
@@ -1681,48 +1840,18 @@ build_generate_dir (BasicAutotoolsPlugin *plugin, const gchar *dirname, const gc
 											   CHOOSE_COMMAND (plugin, AUTORECONF),
 											   args);
 	}
-	build_program_set_callback (prog, build_configure_after_autogen, g_strdup (args));
+	pack->args = g_strdup (args);
+	pack->func = func;
+	pack->name = g_strdup (name);
+	build_program_set_callback (prog, build_configure_after_autogen, pack);
 	
 	context = build_save_and_execute_command (plugin, prog, TRUE, NULL);
 	
 	return context;
 }
 
-
-
-/* User actions
- *---------------------------------------------------------------------------*/
-
-/* Main menu */
 static void
-on_build_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
-{
-	if (plugin->project_root_dir)
-	{
-		build_build_file_or_dir (plugin, plugin->project_root_dir, NULL, NULL, NULL);
-	}
-}
-
-static void
-on_install_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
-{
-	if (plugin->project_root_dir)
-	{
-		build_install_dir (plugin, plugin->project_root_dir, NULL);
-	}
-}
-
-static void
-on_clean_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
-{
-	if (plugin->project_root_dir)
-	{
-		build_clean_dir (plugin, plugin->project_root_dir, NULL);
-	}
-}
-
-static void
-on_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
+build_configure_dialog (BasicAutotoolsPlugin *plugin, BuildFunc func, const gchar *name)
 {
 	GtkWindow *parent;
 	gboolean run_autogen = FALSE;
@@ -1755,20 +1884,101 @@ on_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		
 		if (run_autogen)
 		{
-			build_generate_dir (plugin, build_dir, args);
+			build_generate_dir (plugin, build_dir, args, func, name);
 		}
 		else
 		{
-			build_configure_dir (plugin, build_dir, args);
+			build_configure_dir (plugin, build_dir, args, func, name);
 		}
 		g_free (build_dir);
 	}
+	
+}
+
+/* Run configure if needed and then the build command */
+static void
+build_configure_and_build (BasicAutotoolsPlugin *plugin, BuildFunc func, const gchar *name)
+{
+	const gchar *src_dir;
+	gchar *dir = NULL;
+	gchar *build_dir;
+	gboolean has_makefile;
+
+	/* Find source directory */
+	if (name == NULL)
+	{
+		/* Could be NULL for global project command */
+		src_dir = plugin->project_root_dir;
+	}
+	else if (g_file_test (name, G_FILE_TEST_IS_DIR))
+	{
+		src_dir = name;
+	}
+	else
+	{
+		dir = g_path_get_dirname (name);
+		src_dir = dir;
+	}
+
+	/* Get build directory and check for makefiles */
+	build_dir = build_dir_from_source (plugin, src_dir);
+	has_makefile = directory_has_makefile (build_dir);
+	g_free (build_dir);
+
+	if (!has_makefile)
+	{
+		/* Run configure first */
+		build_configure_dialog (plugin, func, name);
+	}
+	else
+	{
+		/* Some build functions have less arguments but
+		 * it is not a problem in C */
+		func (plugin, name, NULL, NULL, NULL);
+	}
+}
+
+/* User actions
+ *---------------------------------------------------------------------------*/
+
+/* Main menu */
+static void
+on_build_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
+{
+	if (plugin->project_root_dir)
+	{
+		build_configure_and_build (plugin, build_build_file_or_dir, plugin->project_root_dir);
+	}
+}
+
+static void
+on_install_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
+{
+	if (plugin->project_root_dir)
+	{
+		build_configure_and_build (plugin, build_install_dir,  plugin->project_root_dir);
+	}
+}
+
+static void
+on_clean_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
+{
+	if (plugin->project_root_dir)
+	{
+		build_clean_dir (plugin, plugin->project_root_dir, NULL);
+	}
+}
+
+static void
+on_configure_project (GtkAction *action, BasicAutotoolsPlugin *plugin)
+{
+	build_configure_dialog (plugin, NULL, NULL);
 }
 
 static void
 on_build_tarball (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
-	build_tarball (plugin);
+	build_configure_and_build (plugin, (BuildFunc) build_tarball, NULL);	
 }
 
 static void
@@ -1776,9 +1986,9 @@ on_build_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
-		
-		build_build_file_or_dir (plugin, dirname, NULL, NULL, NULL);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
+
+		build_configure_and_build (plugin, build_build_file_or_dir, dirname);
 		
 		g_free (dirname);
 	}
@@ -1789,9 +1999,9 @@ on_install_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
 		
-		build_install_dir (plugin, dirname, NULL);
+		build_configure_and_build (plugin, build_install_dir,  dirname);
 		
 		g_free (dirname);
 	}
@@ -1802,7 +2012,7 @@ on_clean_module (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		gchar *dirname = g_dirname (plugin->current_editor_filename);
+		gchar *dirname = g_path_get_dirname (plugin->current_editor_filename);
 		
 		build_clean_dir (plugin, dirname, NULL);
 		
@@ -1815,7 +2025,7 @@ on_compile_file (GtkAction *action, BasicAutotoolsPlugin *plugin)
 {
 	if (plugin->current_editor_filename)
 	{
-		build_compile_file (plugin, plugin->current_editor_filename);
+		build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->current_editor_filename);
 	}
 }
 
@@ -1858,7 +2068,7 @@ fm_compile (GtkAction *action, BasicAutotoolsPlugin *plugin)
 
 	if (plugin->fm_current_filename)
 	{
-		build_compile_file (plugin, plugin->fm_current_filename);
+		build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->fm_current_filename);
 	}
 }
 
@@ -1873,8 +2083,8 @@ fm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 		dir = g_strdup (plugin->fm_current_filename);
 	else
 		dir = g_path_get_dirname (plugin->fm_current_filename);
-	
-	build_build_file_or_dir (plugin, dir, NULL, NULL, NULL);
+
+	build_configure_and_build (plugin, build_build_file_or_dir, dir);
 	
 	g_free (dir);
 }
@@ -1891,7 +2101,7 @@ fm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	else
 		dir = g_path_get_dirname (plugin->fm_current_filename);
 	
-	build_install_dir (plugin, dir, NULL);
+	build_configure_and_build (plugin, build_install_dir,  dir);
 	
 	g_free (dir);
 }
@@ -1921,7 +2131,7 @@ pm_compile (GtkAction *action, BasicAutotoolsPlugin *plugin)
 
 	if (plugin->pm_current_filename)
 	{
-		build_compile_file (plugin, plugin->pm_current_filename);
+		build_configure_and_build (plugin, (BuildFunc) build_compile_file, plugin->pm_current_filename);
 	}
 }
 
@@ -1937,7 +2147,7 @@ pm_build (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	else
 		dir = g_path_get_dirname (plugin->pm_current_filename);
 
-	build_build_file_or_dir (plugin, dir, NULL, NULL, NULL);
+	build_configure_and_build (plugin, build_build_file_or_dir, dir);
 
 	g_free (dir);
 }
@@ -1954,7 +2164,7 @@ pm_install (GtkAction *action, BasicAutotoolsPlugin *plugin)
 	else
 		dir = g_path_get_dirname (plugin->pm_current_filename);
 	
-	build_install_dir (plugin, dir, NULL);
+	build_configure_and_build (plugin, build_install_dir,  dir);
 	
 	g_free (dir);
 }
@@ -2017,8 +2227,8 @@ static GtkActionEntry build_actions[] =
 		N_("_Build"), NULL, NULL, NULL
 	},
 	{
-		"ActionBuildBuildProject", NULL,
-		N_("_Build Project"), "<shift>F11",
+		"ActionBuildBuildProject", GTK_STOCK_CONVERT,
+		N_("_Build Project"), "<shift>F7",
 		N_("Build whole project"),
 		G_CALLBACK (on_build_project)
 	},
@@ -2047,8 +2257,8 @@ static GtkActionEntry build_actions[] =
 		G_CALLBACK (on_build_tarball)
 	},
 	{
-		"ActionBuildBuildModule", GTK_STOCK_EXECUTE,
-		N_("_Build Module"), "F11",
+		"ActionBuildBuildModule", ANJUTA_STOCK_BUILD,
+		N_("_Build Module"), "F7",
 		N_("Build module associated with current file"),
 		G_CALLBACK (on_build_module)
 	},
@@ -2065,7 +2275,7 @@ static GtkActionEntry build_actions[] =
 		G_CALLBACK (on_clean_module)
 	},
 	{
-		"ActionBuildCompileFile", GTK_STOCK_CONVERT,
+		"ActionBuildCompileFile", NULL,
 		N_("Co_mpile File"), "F9",
 		N_("Compile current editor file"),
 		G_CALLBACK (on_compile_file)
@@ -2150,77 +2360,6 @@ static GtkActionEntry build_popup_actions[] =
 	}
 };
 
-static gboolean
-directory_has_makefile (const gchar *dirname)
-{
-	gchar *makefile;
-	gboolean makefile_exists;
-	
-	makefile_exists = TRUE;	
-	makefile = g_build_filename (dirname, "Makefile", NULL);
-	if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
-	{
-		g_free (makefile);
-		makefile = g_build_filename (dirname, "makefile", NULL);
-		if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
-		{
-			g_free (makefile);
-			makefile = g_build_filename (dirname, "MAKEFILE", NULL);
-			if (!g_file_test (makefile, G_FILE_TEST_EXISTS))
-			{
-				makefile_exists = FALSE;
-			}
-		}
-	}
-	g_free (makefile);
-	return makefile_exists;
-}
-
-static gboolean
-directory_has_file (const gchar *dirname, const gchar *filename)
-{
-	gchar *filepath;
-	gboolean exists;
-	
-	exists = TRUE;	
-	filepath = g_build_filename (dirname, filename, NULL);
-	if (!g_file_test (filepath, G_FILE_TEST_EXISTS))
-		exists = FALSE;
-	
-	g_free (filepath);
-	return exists;
-}
-
-static gchar*
-escape_label (const gchar *str)
-{
-	GString *ret;
-	const gchar *iter;
-	
-	ret = g_string_new ("");
-	iter = str;
-	while (*iter != '\0')
-	{
-		if (*iter == '_')
-		{
-			ret = g_string_append (ret, "__");
-			iter++;
-		}
-		else
-		{
-			const gchar *start;
-			const gchar *end;
-			
-			start = iter;
-			iter = g_utf8_next_char (iter);
-			end = iter;
-			
-			ret = g_string_append_len (ret, start, end - start);
-		}
-	}
-	return g_string_free (ret, FALSE);
-}
-
 static void
 update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 {
@@ -2234,7 +2373,7 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (bb_plugin)->shell, NULL);
 	
-	DEBUG_PRINT ("Updating module UI");
+	DEBUG_PRINT ("%s", "Updating module UI");
 
 	has_file = bb_plugin->current_editor_filename != NULL;
 	if (has_file)
@@ -2242,12 +2381,12 @@ update_module_ui (BasicAutotoolsPlugin *bb_plugin)
 		gchar *dirname;
 		gchar *build_dirname;
 
-		dirname = g_dirname (bb_plugin->current_editor_filename);
+		dirname = g_path_get_dirname (bb_plugin->current_editor_filename);
 		build_dirname = build_dir_from_source (bb_plugin, dirname);
 		
-		module = escape_label (g_basename (dirname));
-		filename = escape_label (g_basename (bb_plugin->current_editor_filename));
-		has_makefile = directory_has_makefile (build_dirname);
+		module = escape_label (g_path_get_basename (dirname));
+		filename = escape_label (g_path_get_basename (bb_plugin->current_editor_filename));
+		has_makefile = directory_has_makefile (build_dirname) || project_has_configure (bb_plugin);
 		g_free (build_dirname);
 		g_free (dirname);
 	}
@@ -2293,10 +2432,10 @@ update_project_ui (BasicAutotoolsPlugin *bb_plugin)
 	gboolean has_makefile;
 	gboolean has_project;
 	
-	DEBUG_PRINT ("Updating project UI");
+	DEBUG_PRINT ("%s", "Updating project UI");
 	
 	has_project = bb_plugin->project_root_dir != NULL;
-	has_makefile = has_project && directory_has_makefile (bb_plugin->project_build_dir);
+	has_makefile = has_project && (directory_has_makefile (bb_plugin->project_build_dir) || project_has_configure (bb_plugin));
 
 	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (bb_plugin)->shell, NULL);
 	action = anjuta_ui_get_action (ui, "ActionGroupBuild",
@@ -2376,6 +2515,15 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 		name = build_configuration_get_name (cfg);
 		anjuta_session_set_string (session, "Build", "Selected Configuration", name);
 	}
+	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
+	{
+		gchar *key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
+
+		anjuta_session_set_string (session, "Build",
+			       key,
+			       build_configuration_get_args(cfg));
+		g_free (key);
+	}
 }
 
 static void
@@ -2384,7 +2532,8 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 {
 	GList *configurations;
 	gchar *selected;
-				
+	BuildConfiguration *cfg;
+
 	if (phase != ANJUTA_SESSION_PHASE_NORMAL)
 		return;
 	
@@ -2399,6 +2548,19 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase,
 	build_configuration_list_select (plugin->configurations, selected);
 	g_free (selected);
 
+	for (cfg = build_configuration_list_get_first (plugin->configurations); cfg != NULL; cfg = build_configuration_next (cfg))
+	{
+		gchar *key = g_strconcat("BuildArgs/", build_configuration_get_name (cfg), NULL);
+		gchar *args = anjuta_session_get_string (session, "Build",
+				key);
+		g_free (key);
+		if (args != NULL)
+		{
+			build_configuration_set_args (cfg, args);
+			g_free (args);
+		}
+	}
+
 	build_project_configured (G_OBJECT (plugin), NULL, NULL, NULL);
 }
 
@@ -2432,7 +2594,7 @@ value_added_fm_current_file (AnjutaPlugin *plugin, const char *name,
 		dirname = g_strdup (filename);
 	else
 		dirname = g_path_get_dirname (filename);
-	makefile_exists = directory_has_makefile (dirname);
+	makefile_exists = directory_has_makefile (dirname) || project_has_configure (ba_plugin);
 	g_free (dirname);
 	
 	if (!makefile_exists)
@@ -2474,12 +2636,9 @@ value_added_pm_current_uri (AnjutaPlugin *plugin, const char *name,
 	const gchar *uri;
 	gchar *dirname, *filename;
 	gboolean makefile_exists, is_dir;
-	GFile* file;
 	
 	uri = g_value_get_string (value);
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	g_return_if_fail (filename != NULL);
 	
 	/*
@@ -2499,7 +2658,7 @@ value_added_pm_current_uri (AnjutaPlugin *plugin, const char *name,
 		dirname = g_strdup (filename);
 	else
 		dirname = g_path_get_dirname (filename);
-	makefile_exists = directory_has_makefile (dirname);
+	makefile_exists = directory_has_makefile (dirname) || project_has_configure (ba_plugin);
 	g_free (dirname);
 	
 	if (!makefile_exists)
@@ -2754,6 +2913,20 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	update_module_ui (ba_plugin);
 }
 
+static void
+register_stock_icons (AnjutaPlugin *plugin)
+{	
+	static gboolean registered = FALSE;
+
+	if (registered)
+		return;
+	registered = TRUE;
+
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_BUILD, ANJUTA_STOCK_BUILD);
+	END_REGISTER_ICON;
+}
+
 static gboolean
 activate_plugin (AnjutaPlugin *plugin)
 {
@@ -2761,6 +2934,10 @@ activate_plugin (AnjutaPlugin *plugin)
 	static gboolean initialized = FALSE;
 	BasicAutotoolsPlugin *ba_plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (plugin);
 	
+	if (!initialized)
+	{
+		register_stock_icons (plugin);
+	}
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
 	g_signal_connect (plugin->shell, "save-session",
@@ -2780,9 +2957,11 @@ activate_plugin (AnjutaPlugin *plugin)
 											GETTEXT_PACKAGE, TRUE, plugin);
 	ba_plugin->build_popup_action_group = 
 		anjuta_ui_add_action_group_entries (ui,
-											"ActionGroupPopupBuild",
-											_("Build popup commands"),
-											build_popup_actions,
+			       	"ActionGroupPopupBuild",
+				/* Translators: This is a group of build
+				 * commands which appears in pop up menus */
+			       	_("Build popup commands"),
+			       	build_popup_actions,
 											sizeof(build_popup_actions)/sizeof(GtkActionEntry),
 											GETTEXT_PACKAGE, FALSE, plugin);
 	/* Add UI */
@@ -2984,7 +3163,7 @@ ibuildable_install (IAnjutaBuildable *manager, const gchar *directory,
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
 	
-	build_install_dir (plugin, directory, err);
+	build_install_dir (plugin, directory, NULL, NULL, err);
 }
 
 static void
@@ -2993,7 +3172,7 @@ ibuildable_configure (IAnjutaBuildable *manager, const gchar *directory,
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
 	
-	build_configure_dir (plugin, directory, NULL);
+	build_configure_dir (plugin, directory, NULL, NULL, NULL);
 }
 
 static void
@@ -3002,7 +3181,7 @@ ibuildable_generate (IAnjutaBuildable *manager, const gchar *directory,
 {
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (manager);
 
-	build_generate_dir (plugin, directory, NULL);
+	build_generate_dir (plugin, directory, NULL, NULL, NULL);
 }
 
 static void
@@ -3047,7 +3226,7 @@ ifile_open (IAnjutaFile *manager, GFile* file,
 static GFile*
 ifile_get_file (IAnjutaFile *manager, GError **err)
 {
-	DEBUG_PRINT ("Unsupported operation");
+	DEBUG_PRINT ("%s", "Unsupported operation");
 	return NULL;
 }
 
@@ -3070,11 +3249,8 @@ ibuilder_is_built (IAnjutaBuilder *builder, const gchar *uri,
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
 	BuildContext *context;
 	gchar *filename;
-	GFile* file;
 
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	if (filename == NULL) return NULL;
 	
 	context = build_is_file_built (plugin, filename, callback, user_data, err);
@@ -3092,11 +3268,8 @@ ibuilder_build (IAnjutaBuilder *builder, const gchar *uri,
 	BasicAutotoolsPlugin *plugin = ANJUTA_PLUGIN_BASIC_AUTOTOOLS (builder);
 	BuildContext *context;
 	gchar *filename;
-	GFile* file;
 	
-	file = g_file_new_for_uri (uri);
-	filename = g_file_get_path (file);
-	g_object_unref (file);
+	filename = anjuta_util_get_local_path_from_uri (uri);
 	if (filename == NULL) return NULL;
 	
 	context = build_build_file_or_dir (plugin, filename, callback, user_data, err);
