@@ -18,7 +18,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #include <config.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libanjuta/anjuta-ui.h>
 #include <libanjuta/anjuta-utils.h>
 
@@ -29,7 +28,6 @@
 #include <libanjuta/interfaces/ianjuta-editor-line-mode.h>
 #include <libanjuta/interfaces/ianjuta-editor-view.h>
 #include <libanjuta/interfaces/ianjuta-editor-folds.h>
-#include <libanjuta/interfaces/ianjuta-bookmark.h>
 #include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-file-savable.h>
 #include <libanjuta/interfaces/ianjuta-print.h>
@@ -46,6 +44,7 @@
 #include "plugin.h"
 #include "file_history.h"
 #include "search-box.h"
+#include "anjuta-bookmarks.h"
 
 static IAnjutaDocument *
 get_current_document (gpointer user_data)
@@ -185,12 +184,20 @@ on_close_file_activate (GtkAction *action, gpointer user_data)
 	{
 		GtkWidget *parent;
 		gchar *uri;
+		GFile* file;
 		AnjutaSavePrompt *save_prompt;
 		
 		parent = gtk_widget_get_toplevel (GTK_WIDGET (doc));
 		/* Prompt for unsaved data */
 		save_prompt = anjuta_save_prompt_new (GTK_WINDOW (parent));
-		uri = ianjuta_file_get_uri (IANJUTA_FILE (doc), NULL);
+		file = ianjuta_file_get_file (IANJUTA_FILE (doc), NULL);
+		if (file)
+		{
+			uri = g_file_get_uri (file);
+			g_object_unref (file);
+		}
+		else
+			uri = NULL;
 		/* NULL uri ok */
 		anjuta_save_prompt_add_item (save_prompt,
 									 ianjuta_document_get_filename (doc, NULL),
@@ -256,40 +263,45 @@ on_reload_file_activate (GtkAction *action, gpointer user_data)
 	gchar *msg;
 	GtkWidget *dialog;
 	GtkWidget *parent;
+	gint reply = GTK_RESPONSE_YES;
 	
 	doc = get_current_document (user_data);
 	if (doc == NULL)
 		return;
 
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (doc));
-	msg = g_strdup_printf (
-	_("Are you sure you want to reload '%s'?\nAny unsaved changes will be lost."),
-							ianjuta_document_get_filename (doc, NULL));
-
-	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
-									 GTK_DIALOG_DESTROY_WITH_PARENT,
-									 GTK_MESSAGE_QUESTION,
-									 GTK_BUTTONS_NONE, msg);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-						   GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
-	anjuta_util_dialog_add_button (GTK_DIALOG (dialog), _("_Reload"),
-							  GTK_STOCK_REVERT_TO_SAVED,
-							  GTK_RESPONSE_YES);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-									 GTK_RESPONSE_NO);
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+	if (IANJUTA_IS_FILE_SAVABLE (doc) && ianjuta_file_savable_is_dirty (IANJUTA_FILE_SAVABLE (doc), NULL))
 	{
-		gchar* uri;
-		uri = ianjuta_file_get_uri (IANJUTA_FILE (doc), NULL);
-		if (uri)
+		msg = g_strdup_printf (
+		_("Are you sure you want to reload '%s'?\nAny unsaved changes will be lost."),
+								ianjuta_document_get_filename (doc, NULL));
+
+		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+										 GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_NONE, "%s", msg);
+		gtk_dialog_add_button (GTK_DIALOG (dialog),
+							   GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
+		anjuta_util_dialog_add_button (GTK_DIALOG (dialog), _("_Reload"),
+								  GTK_STOCK_REVERT_TO_SAVED,
+								  GTK_RESPONSE_YES);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+										 GTK_RESPONSE_NO);
+		reply = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		g_free (msg);
+	}
+		
+	if (reply == GTK_RESPONSE_YES)
+	{
+		GFile* file;
+		file = ianjuta_file_get_file (IANJUTA_FILE (doc), NULL);
+		if (file)
 		{
-			ianjuta_file_open (IANJUTA_FILE (doc), uri, NULL);
-			/* FIXME: anjuta_update_title (); */
-			g_free (uri);
+			ianjuta_file_open (IANJUTA_FILE (doc), file, NULL);
+			g_object_unref(file);
 		}
 	}
-	gtk_widget_destroy (dialog);
-	g_free (msg);
 }
 
 void
@@ -415,16 +427,6 @@ on_editor_command_select_all_activate (GtkAction *action, gpointer user_data)
 	}
 }
 
-void
-on_editor_command_select_to_brace_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_editor_selection_select_to_brace (IANJUTA_EDITOR_SELECTION (doc), NULL);
-}
-
 void
 on_editor_command_select_block_activate (GtkAction *action, gpointer user_data)
 {
@@ -599,68 +601,6 @@ on_editor_command_toggle_fold_activate (GtkAction *action, gpointer user_data)
 		ianjuta_editor_folds_toggle_current (IANJUTA_EDITOR_FOLDS (doc), NULL);
 }
 
-void
-on_editor_command_bookmark_toggle_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_toggle (IANJUTA_BOOKMARK (doc),
-								ianjuta_editor_get_lineno (IANJUTA_EDITOR (doc), NULL),
-								FALSE, NULL);
-}
-
-void
-on_editor_command_bookmark_first_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_first (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_next_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_next (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_prev_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_previous (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_last_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_last (IANJUTA_BOOKMARK (doc), NULL);
-}
-
-void
-on_editor_command_bookmark_clear_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-
-	doc = get_current_document (user_data);
-	if (doc)
-		ianjuta_bookmark_clear_all (IANJUTA_BOOKMARK (doc), NULL);
-}
-
 void
 on_transform_eolchars1_activate (GtkAction *action, gpointer user_data)
 {
@@ -876,134 +816,6 @@ on_force_hilite_activate (GtkWidget *menuitem, gpointer user_data)
 	}
 }
 
-/* Gets the swapped (c/h) file names */
-static gchar*
-get_swapped_filename (const gchar* filename)
-{
-	size_t len;
-	gchar *newfname;
-	GnomeVFSURI *vfs_uri;
-
-	g_return_val_if_fail (filename != NULL, NULL);
-
-	len = strlen (filename);
-	newfname = g_malloc (len+5); /* to be on the safer side */
-	while (len)
-	{
-		if (filename[len] == '.')
-			break;
-		len--;
-	}
-	len++;
-	strcpy (newfname, filename);
-	if (strncasecmp (&filename[len], "h", 1) == 0)
-	{
-		strcpy (&newfname[len], "cc");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "cpp");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "cxx");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "c");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{ 
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-	}
-	else if (strncasecmp (&filename[len], "c", 1)==0 )
-	{
-		strcpy (&newfname[len], "h");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{ 
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "hh");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "hxx");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		strcpy (&newfname[len], "hpp");
-		vfs_uri = gnome_vfs_uri_new (newfname);
-		if (gnome_vfs_uri_exists (vfs_uri))
-		{
-			gnome_vfs_uri_unref (vfs_uri);
-			return newfname;
-		}
-		gnome_vfs_uri_unref (vfs_uri);
-	}
-	g_free (newfname);	
-	return NULL;
-}
-
-void
-on_swap_activate (GtkAction *action, gpointer user_data)
-{
-	IAnjutaDocument *doc;
-	AnjutaDocman *docman;
-	DocmanPlugin *plugin;
-
-	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
-	docman = ANJUTA_DOCMAN (plugin->docman);
-	doc = anjuta_docman_get_current_document (docman);
-	if (doc)
-	{
-		gchar *uri;
-		uri = ianjuta_file_get_uri (IANJUTA_FILE (doc), NULL);
-		if (uri)
-		{
-			gchar *newfname;
-			newfname = get_swapped_filename (uri);
-			g_free (uri);
-			if (newfname)
-			{
-				anjuta_docman_goto_uri_line (docman, newfname, -1);
-				g_free (newfname);
-			}
-		}
-	}
-}
-
 void
 on_show_search (GtkAction *action, gpointer user_data)
 {
@@ -1055,24 +867,92 @@ on_editor_remove_view_activate (GtkAction *action, gpointer user_data)
 		ianjuta_editor_view_remove_current (IANJUTA_EDITOR_VIEW (doc), NULL);
 }
 
-void on_next_document (GtkAction *action, gpointer user_data)
+void
+on_next_document (GtkAction *action, gpointer user_data)
 {
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
-	
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
-								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) + 1);
+	GtkNotebook* notebook = GTK_NOTEBOOK (docman); 
+	gint cur_page = gtk_notebook_get_current_page(notebook);
+	if (cur_page <
+		gtk_notebook_get_n_pages(notebook) - 1)
+		cur_page++;
+	else
+		cur_page = 0;
+
+	gtk_notebook_set_current_page (notebook,
+								   cur_page);
 }
 
-void on_previous_document (GtkAction *action, gpointer user_data)
+void 
+on_previous_document (GtkAction *action, gpointer user_data)
 {
 	AnjutaDocman *docman;
 	DocmanPlugin *plugin;
 	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
 	docman = ANJUTA_DOCMAN (plugin->docman);
+	GtkNotebook* notebook = GTK_NOTEBOOK (docman); 
+	gint cur_page = gtk_notebook_get_current_page(notebook);
+	if (cur_page > 0)
+		cur_page--;
+	else
+		cur_page = -1; /* last_page */
 	
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (docman),
-								   gtk_notebook_get_current_page (GTK_NOTEBOOK(docman)) - 1);
+	gtk_notebook_set_current_page (notebook,
+								   cur_page);
+}
+
+void
+on_bookmark_add_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_add (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							  ianjuta_editor_get_lineno (editor, NULL), NULL, TRUE);
+	}
+}
+
+void 
+on_bookmark_next_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_next (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							  ianjuta_editor_get_lineno (editor, NULL));
+	}
+}
+
+void
+on_bookmark_prev_activate (GtkAction *action, gpointer user_data)
+{
+	IAnjutaDocument *doc;
+	DocmanPlugin *plugin;
+	doc = get_current_document (user_data);
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	if (doc && IANJUTA_IS_EDITOR(doc))
+	{
+		IAnjutaEditor* editor = IANJUTA_EDITOR(doc);
+		anjuta_bookmarks_prev (ANJUTA_BOOKMARKS (plugin->bookmarks), editor, 
+							   ianjuta_editor_get_lineno (editor, NULL));
+	}
+}
+
+void
+on_bookmarks_clear_activate (GtkAction *action, gpointer user_data)
+{
+	DocmanPlugin *plugin;
+	plugin = ANJUTA_PLUGIN_DOCMAN (user_data);
+	anjuta_bookmarks_clear (ANJUTA_BOOKMARKS(plugin->bookmarks));
 }
