@@ -122,7 +122,8 @@ on_menu_insert_macro (GtkAction * action, MacroPlugin * plugin)
 		return;
 	
 	GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	GtkWidget* entry = gtk_entry_new_with_max_length(1);
+	GtkWidget* entry = gtk_entry_new();
+	gtk_entry_set_max_length (GTK_ENTRY(entry), 1);
 	GtkWidget* label = gtk_label_new_with_mnemonic(_("Press macro shortcut..."));
 	GtkWidget* hbox = gtk_hbox_new (FALSE, 0);	
 	
