@@ -39,7 +39,6 @@
 
 #include <gnome.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-shell.h>
@@ -63,7 +62,6 @@
 */
 
 #include "search-replace_backend.h"
-#include "tm_tagmanager.h"
 
 /* Information about a matched substring */
 typedef struct _MatchSubStr
@@ -104,10 +102,8 @@ file_buffer_free (FileBuffer *fb)
 {
 	if (fb)
 	{
-		if (fb->path)
-			g_free(fb->path);
-		if (fb->uri)
-			g_free (fb->uri);
+		if (fb->file)
+			g_object_unref (fb->file);
 		if (fb->buf)
 			g_free(fb->buf);
 		if (fb->lines)
@@ -120,29 +116,21 @@ file_buffer_free (FileBuffer *fb)
 FileBuffer *
 file_buffer_new_from_te (IAnjutaEditor *te)
 {
-	FileBuffer *fb;
-	GFile* file;
-	gchar* path;
+	FileBuffer *fb = NULL;
+	gint len;
 	
 	g_return_val_if_fail(te, NULL);
-	fb = g_new0(FileBuffer, 1);
-	fb->type = FB_EDITOR;
-	fb->te = te;
 	
-	file = ianjuta_file_get_file(IANJUTA_FILE(te), NULL);
-	path = g_file_get_path (file);
-	if (path)
+	len = ianjuta_editor_get_length(te, NULL);
+	if (len != 0)
 	{
-		fb->path = tm_get_real_path(path);
-		g_free (path);
+		fb = g_new0(FileBuffer, 1);
+		fb->type = FB_EDITOR;
+		fb->te = te;
+		fb->file = ianjuta_file_get_file(IANJUTA_FILE(te), NULL);
+		fb->buf = ianjuta_editor_get_text_all (fb->te, NULL);
+		fb->len = len;
 	}
-	fb->uri = g_file_get_uri (file);
-	fb->len = ianjuta_editor_get_length(te, NULL);
-	fb->buf = ianjuta_editor_get_text_all (fb->te, NULL);
-	fb->pos = ianjuta_editor_get_offset(fb->te, NULL);
-	fb->line = ianjuta_editor_get_lineno(fb->te, NULL);
-	
-	g_object_unref (file);
 	
 	return fb;
 }
@@ -151,89 +139,47 @@ file_buffer_new_from_te (IAnjutaEditor *te)
 #define MAX_VALIDATE 500
 
 FileBuffer *
-file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
+file_buffer_new_from_uri (const gchar *uri)
 {
 	FileBuffer *fb;
-	IAnjutaEditor *te;
+	GFile *file;
 	IAnjutaDocument* doc;
-	char *real_path;
-	GFile* file;
-	char *uri;
-	int i;
-	int lineno;
+	gchar *buffer;
+	gsize length;
 
-	g_return_val_if_fail(path, NULL);
-	real_path = tm_get_real_path(path);
+	g_return_val_if_fail(uri, NULL);
+	
+	
+	file = g_file_new_for_uri (uri);
 	
 	/* There might be an already open TextEditor with this path */
-	file = g_file_new_for_path (real_path);
-	uri = g_file_get_uri (file);
 	doc = ianjuta_document_manager_find_document_with_file (sr->docman,
 														 file, NULL);
-	g_object_unref (file);
-
 	if (doc && IANJUTA_IS_EDITOR (doc))
 	{
+		IAnjutaEditor *te;
+		
 		te = IANJUTA_EDITOR (doc);
-		g_free(real_path);
+		g_object_unref (file);
+
 		return file_buffer_new_from_te(te);
 	}
-	fb = g_new0(FileBuffer, 1);
-	fb->type = FB_FILE;
-	fb->path = real_path;
-	fb->uri = uri;
-	fb->name = strrchr(path, '/');
-	if (fb->name)
-		++ fb->name;
-	else
-		fb->name = fb->path;
-	if (buf && len > 0)
-	{
-		fb->buf = g_new(char, len + 1);
-		memcpy(fb->buf, buf, len);
-		fb->buf[len] = '\0';
-		fb->len = len;
-	}
-	else
-	{
-		struct stat s;
 
-		if ((0 == stat(fb->path, &s)) && (S_ISREG(s.st_mode)))
-		{
-			if ((fb->len = s.st_size) < 0) return NULL;
-			fb->buf = g_new(char, s.st_size + 1);
-			{
-				int total_bytes = 0, bytes_read, fd;
-				if (0 > (fd = open(fb->path, O_RDONLY)))
-				{
-					perror(fb->path);
-					file_buffer_free(fb);
-					return NULL;
-				}
-				while (total_bytes < s.st_size)
-				{
-					if (0 > (bytes_read = read(fd, fb->buf + total_bytes
-					  , s.st_size - total_bytes)))
-					{
-						perror(fb->path);
-						close(fd);
-						file_buffer_free(fb);
-						return NULL;
-					}
-					total_bytes += bytes_read;
-				}
-				close(fd);
-				fb->buf[fb->len] = '\0';
-			}
-		}
+	if (!g_file_load_contents (file, NULL, &buffer, &length, NULL, NULL))
+	{
+		/* Unable to read file */
+		g_object_unref (file);
+		
+		return NULL;
 	}
-	if (!g_utf8_validate (fb->buf, MIN(MAX_VALIDATE, fb->len), NULL))
+	
+	if (!g_utf8_validate (buffer, MIN(MAX_VALIDATE, length), NULL))
 	{
 		const AnjutaEncoding *encoding_used = NULL;
 		gchar* converted_text;
-		guint converted_len;
-		converted_text = anjuta_convert_to_utf8 (fb->buf, 
-												 fb->len, 
+		gsize converted_len;
+		converted_text = anjuta_convert_to_utf8 (buffer, 
+												 length, 
 												 &encoding_used,
 												 &converted_len, 
 												 NULL);
@@ -243,61 +189,66 @@ file_buffer_new_from_path (const char *path, const char *buf, int len, int pos)
 			encoding_used =  
 				anjuta_encoding_get_from_charset("ISO-8859-15");
 			
-			converted_text = anjuta_convert_to_utf8 (fb->buf,
-												  fb->len,
+			converted_text = anjuta_convert_to_utf8 (buffer,
+												  length,
 												  &encoding_used,
 												  &converted_len,
 												  NULL);
 		}
+		g_free (buffer);
+		
 		if (converted_text == NULL)
 		{
 			/* Give up */
-			file_buffer_free(fb);
+			g_object_unref (file);
+			
 			return NULL;
 		}
-		else
-		{
-			g_free (fb->buf);
-			fb->buf = converted_text;
-			fb->len = converted_len;
-		}
+		
+		buffer = converted_text;
+		length = converted_len;
 	}
 	
-	if (pos <= 0 || pos > fb->len)
-	{
-		fb->pos = 0;
-		fb->line = 0;
-	}
-	else
-	{
-		fb->pos = pos;
-		fb->line = 0;
-	}
-	/* First line starts at column 0 */
-	fb->lines = g_list_prepend(fb->lines, GINT_TO_POINTER(0));
-	lineno = 0;
-	for (i=0; i < fb->len; ++i)
-	{
-		if ('\n' == fb->buf[i] && '\0' != fb->buf[i+1])
-		{
-			fb->lines = g_list_prepend(fb->lines, GINT_TO_POINTER(i + 1));
-			if (0 == fb->line && fb->pos > i)
-				fb->line = lineno;
-			++ lineno;
-		}
-	}
-	fb->lines = g_list_reverse(fb->lines);
+	fb = g_new0(FileBuffer, 1);
+	fb->type = FB_FILE;
+	
+	fb->file = file;
+	fb->len = length;
+	fb->buf = buffer;
+	
 	return fb;
 }
 
 static long
-file_buffer_line_from_pos(FileBuffer *fb, int pos)
+file_buffer_line_from_pos(FileBuffer *fb, gint pos)
 {
-	GList *tmp;
-	int lineno = -1;
+	gint lineno = -1;
+	
 	g_return_val_if_fail(fb && pos >= 0, 1);
+	
 	if (FB_FILE == fb->type)
 	{
+		GList *tmp;
+		
+		if (fb->lines == NULL)
+		{
+			gint i;
+			/* First line starts at column 0 */
+			fb->lines = g_list_prepend(fb->lines, GINT_TO_POINTER(0));
+			lineno = 0;
+			for (i=0; i < fb->len; ++i)
+			{
+				if ('\n' == fb->buf[i] && '\0' != fb->buf[i+1])
+				{
+					fb->lines = g_list_prepend(fb->lines, GINT_TO_POINTER(i + 1));
+					if (0 == fb->line && fb->pos > i)
+						fb->line = lineno;
+					++ lineno;
+				}
+			}
+			fb->lines = g_list_reverse(fb->lines);
+		}
+		
 		for (tmp = fb->lines; tmp; tmp = g_list_next(tmp))
 		{
 			if (pos < GPOINTER_TO_INT(tmp->data))
@@ -309,9 +260,13 @@ file_buffer_line_from_pos(FileBuffer *fb, int pos)
 	else if (FB_EDITOR == fb->type)
 	{
 		IAnjutaIterable *position;
-		position = ianjuta_editor_get_position_from_offset (fb->te, pos, NULL);
+		gint offset;
+		
+		offset = g_utf8_strlen (fb->buf, pos);
+		position = ianjuta_editor_get_position_from_offset (fb->te, offset, NULL);
 		lineno = ianjuta_editor_get_line_from_position (fb->te, position, NULL);
 		g_object_unref (position);
+		
 		return lineno;
 	}
 	else
@@ -326,31 +281,11 @@ file_match_line_from_pos(FileBuffer *fb, int pos)
 	g_return_val_if_fail(fb && pos >= 0, NULL);
 
 	for (i= pos+1; ((fb->buf[i] != '\n') && (fb->buf[i] != '\0')); i++, length++);
-	for (i= pos-1; (fb->buf[i] != '\n') && (i >= 0); i--, length++);
+	for (i= pos -1; (fb->buf[i] != '\n') && (i >= 0); i--, length++);
 	
 	return g_strndup (fb->buf + i + 1, length);
 }
 
-/* Generate a list of files to search in. Call with start = TRUE and
-** top_dir = sf->top_dir. This is used when the search range is specified as
-SR_FILES */
-static GList *
-create_search_files_list(SearchFiles *sf, const char *top_dir)
-{
-	TMFileEntry *entry;
-	GList *files;
-
-	g_return_val_if_fail(sf && top_dir, NULL);
-	entry = tm_file_entry_new(top_dir, NULL, sf->recurse, sf->match_files
-	  , sf->ignore_files, sf->match_dirs, sf->ignore_dirs
-	  , sf->ignore_hidden_files, sf->ignore_hidden_dirs);
-	if (!entry)
-		return NULL;
-	files = tm_file_entry_list(entry, NULL);
-	tm_file_entry_free(entry);
-	return files;
-}
-
 /* Get a list of all project files */
 static GList *
 get_project_file_list(void)
@@ -380,12 +315,8 @@ get_project_file_list(void)
 	
 			while (node)
 			{
-				gchar *file_path;
-		
 				uri = (const gchar *)node->data;
-				file_path = gnome_vfs_get_local_path_from_uri (uri);
-				if (file_path)
-				files = g_list_prepend (files, file_path);
+				files = g_list_prepend (files, g_strdup (uri));
 				node = g_list_next (node);
 			}
 			files = g_list_reverse (files);
@@ -404,27 +335,29 @@ isawordchar (gunichar c)
 }
 
 static gboolean
-extra_match (FileBuffer *fb, gchar* begin, gchar* end, SearchExpression *s)
+extra_match (gboolean at_start, gchar* begin, gchar* end, SearchExpression *s)
 {
 	gunichar b, e;
 	
 	b = g_utf8_get_char (g_utf8_prev_char (begin));
 	e = g_utf8_get_char (end);
+	if (g_unichar_ismark(e))
+		return FALSE;  /* matched character continues past match end */
 	
 	if (s->whole_line)
-		if ((fb->pos == 0 || b == '\n' || b == '\r') &&
+		if ((at_start || b == '\n' || b == '\r') &&
 			(e == '\0'	|| e == '\n' || e == '\r'))
 			return TRUE;
 		else
 			return FALSE;
 	else if (s->whole_word)
-		if ((fb->pos ==0 || !isawordchar(b)) && 
+		if ((at_start || !isawordchar(b)) && 
 			(e=='\0' || !isawordchar(e)))
 			return TRUE;
 		else
 			return FALSE;
 	else if (s->word_start)
-		if (fb->pos ==0 || !isawordchar(b))
+		if (at_start || !isawordchar(b))
 			return TRUE;
 		else
 			return FALSE;	
@@ -432,199 +365,269 @@ extra_match (FileBuffer *fb, gchar* begin, gchar* end, SearchExpression *s)
 		return TRUE;
 }
 
-/* Returns the next match in the passed buffer. The search expression should
-   be pre-compiled. The returned pointer should be freed with match_info_free()
-   when no longer required. */
-MatchInfo *
-get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+static MatchInfo *
+get_next_regex_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
 {
 	MatchInfo *mi = NULL;
-
-	g_return_val_if_fail(fb && s, NULL);
+	GMatchInfo* match_info;
 	
-	if (s->regex)
+	if (s->regex_info == NULL)
 	{
-		GMatchInfo* match_info;
-		if (s->regex_info == NULL)
+		GError* error = NULL;
+		GRegexCompileFlags compile_flags = 0;
+		GRegexMatchFlags match_flags = 0;
+	
+		match_flags |= G_REGEX_MATCH_NOTEMPTY;
+		if (!s->match_case)
 		{
-			GError* error = NULL;
-			GRegexCompileFlags compile_flags = 0;
-			GRegexMatchFlags match_flags = 0;
-		
-			match_flags |= G_REGEX_MATCH_NOTEMPTY;
-			if (s->ignore_case)
-			{
-				compile_flags |= G_REGEX_CASELESS;
-			}
-			if (!s->greedy)
-			{
-				compile_flags |= G_REGEX_UNGREEDY;
-			}
-			s->regex_info = g_regex_new (s->search_str, compile_flags,
-										 match_flags, &error);
-			if (error)
-			{
-				anjuta_util_dialog_error (NULL, error->message);
-				g_error_free(error);
-				s->regex_info = NULL;
-				return NULL;
-			}
+			compile_flags |= G_REGEX_CASELESS;
 		}
+		if (!s->greedy)
+		{
+			compile_flags |= G_REGEX_UNGREEDY;
+		}
+		s->regex_info = g_regex_new (s->search_str, compile_flags,
+									 match_flags, &error);
+		if (error)
+		{
+			anjuta_util_dialog_error (NULL, error->message);
+			g_error_free(error);
+			s->regex_info = NULL;
+			return NULL;
+		}
+	}
 
-		g_regex_match_full (s->regex_info, fb->buf, fb->len,
-					   g_utf8_offset_to_pointer (fb->buf, fb->pos) - fb->buf, 
-					   G_REGEX_MATCH_NOTEMPTY, &match_info, NULL);
-		
-		if (g_match_info_matches (match_info))
+	g_regex_match_full (s->regex_info, fb->buf, fb->len,
+				   fb->pos, 
+				   G_REGEX_MATCH_NOTEMPTY, &match_info, NULL);
+	
+	if (g_match_info_matches (match_info))
+	{
+		gint start;
+		gint end;
+		int i;
+		mi = g_new0(MatchInfo, 1);
+		if (g_match_info_fetch_pos (match_info, 0, &start, &end))
 		{
-			gint start;
-			gint end;
-			int i;
-			mi = g_new0(MatchInfo, 1);
-			if (g_match_info_fetch_pos (match_info, 0, &start, &end))
-			{
-				DEBUG_PRINT ("Regex: %d %d", start, end);
-				mi->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
-				mi->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - mi->pos;
-				mi->line = file_buffer_line_from_pos(fb, mi->pos);
-			}
-			for (i = 1; i < g_match_info_get_match_count(match_info); i++) /* Captured subexpressions */
-			{
-				MatchSubStr *ms;
-				ms = g_new0(MatchSubStr, 1);
-				if (g_match_info_fetch_pos (match_info, i, &start, &end))
-				{
-					ms->start = g_utf8_pointer_to_offset (fb->buf, fb->buf + start);
-					ms->len = g_utf8_pointer_to_offset (fb->buf, fb->buf + end) - ms->start;
-				}
-				mi->subs = g_list_prepend(mi->subs, ms);
-			}
-			mi->subs = g_list_reverse(mi->subs);
-			fb->pos = g_utf8_pointer_to_offset (fb->buf, fb->buf + end);
+			DEBUG_PRINT ("Regex: %d %d", start, end);
+			mi->pos = start;
+			mi->len = end - mi->pos;
+			mi->line = file_buffer_line_from_pos(fb, mi->pos);
 		}
+		for (i = 1; i < g_match_info_get_match_count(match_info); i++) /* Captured subexpressions */
+		{
+			MatchSubStr *ms;
+			ms = g_new0(MatchSubStr, 1);
+			if (g_match_info_fetch_pos (match_info, i, &start, &end))
+			{
+				ms->start =start;
+				ms->len = end - ms->start;
+			}
+			mi->subs = g_list_prepend(mi->subs, ms);
+		}
+		mi->subs = g_list_reverse(mi->subs);
+		fb->pos = end;
+	}
+	return mi;
+}
+
+static MatchInfo *match_info(FileBuffer *fb, gchar* match, gchar* match_end,
+							 SearchDirection direction)
+{
+	MatchInfo *mi = g_new0 (MatchInfo, 1);
+	mi->pos = match - fb->buf;
+	mi->len = match_end - match;
+	mi->line = file_buffer_line_from_pos (fb, mi->pos);
+
+	if (direction == SD_BACKWARD)
+		fb->pos = mi->pos;
+	else
+		fb->pos = mi->pos + mi->len;
+	
+	return mi;
+}
+
+/* Get the next match, given that the search string is ASCII (as will nearly
+ * always be the case).  In this case we don't need to perform any UTF-8
+ * normalization, since any bytes with values 0-127 in a UTF-8 string always
+ * actually represent ASCII characters. */
+static MatchInfo *
+get_next_ascii_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	int len = strlen(s->search_str);
+	gint (*compare)(const gchar *, const gchar *, gsize) =
+		s->match_case ? strncmp : g_ascii_strncasecmp;
+	gchar *p = fb->buf + fb->pos;
+	
+	if (direction == SD_BACKWARD)
+	{
+		for (; p >= fb->buf; --p)
+			if (!compare(p, s->search_str, len) &&
+				extra_match(p == fb->buf, p, p + len, s))
+				return match_info(fb, p, p + len, direction);
 	}
 	else
 	{
-		/* Simple string search - this needs to be performance-tuned */
-		gboolean found;
-		gint match_len;
+		for (; *p ; ++p)
+			if (!compare(p, s->search_str, len) &&
+				extra_match(p == fb->buf, p, p + len, s))
+				return match_info(fb, p, p + len, direction);
+	}
+	
+	return NULL;			
+}
 
-		match_len = g_utf8_strlen (s->search_str, -1);
-		if (match_len == 0)
-			return NULL;
+/* Normalize a UTF-8 string, optionally folding case.  Returns NULL if invalid. */
+static gchar* normalize(gchar *s, int len_bytes, gboolean match_case) {
+	gchar *c, *n;
+	
+	if (match_case)
+		return g_utf8_normalize(s, len_bytes, G_NORMALIZE_NFD);
+	
+	c = g_utf8_casefold(s, len_bytes);
+	n = g_utf8_normalize(c, -1, G_NORMALIZE_NFD);
+	g_free(c);
+	return n;
+}
 
-		found = FALSE;
-		if (SD_BACKWARD == direction)
+/* Given a UTF-8 string s, advance past a prefix whose normalized length in
+ * bytes is [normal_bytes]. */
+static gchar* normal_advance(gchar *s, int normal_bytes, gboolean match_case)
+{
+	while (*s && normal_bytes > 0)
+	{
+		gchar *next = g_utf8_next_char(s);
+		gchar *n = normalize(s, next - s, match_case);
+		normal_bytes -= strlen(n);
+		g_free(n);
+		s = next;
+	}
+	return s;
+}
+
+static MatchInfo *
+get_next_utf8_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	gchar* search_key = normalize(s->search_str, -1, s->match_case);
+	gchar* current = fb->buf + fb->pos;
+	gchar* search_buf;
+	gchar* p = NULL;
+	MatchInfo *mi = NULL;
+	int keylen;
+	
+	if (!search_key)
+		return NULL;
+	keylen = strlen(search_key);
+	
+	if (direction == SD_BACKWARD)
+	{
+		search_buf = normalize(fb->buf, current + strlen (s->search_str) - 1 - fb->buf, s->match_case);
+		if (search_buf)
 		{
-			/* Backward matching. */
-			if (s->ignore_case)
+			p = search_buf + strlen(search_buf);
+			while ((p = g_strrstr_len(search_buf, p - search_buf, search_key)) != NULL)
 			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
-				for (; fb->pos >= len; --fb->pos)
-				{
-					gchar* current_caseless = g_utf8_casefold (current, len);
-					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
-						extra_match (fb, current, current + strlen (search_caseless),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_caseless);
-						break;
-					}
-					else
-						current = g_utf8_prev_char (current);
-				}
-				g_free (search_caseless);
+				if (extra_match (p == search_buf, p, p + keylen, s))
+					break;
+				p = p + keylen - 1;
 			}
-			else
+		}
+	}
+	else
+	{  /* forward search */
+		search_buf = normalize(current, -1, s->match_case);
+		if (search_buf)
+		{
+			p = search_buf;
+			while ((p = strstr(p, search_key)) != NULL)
 			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_key = g_utf8_collate_key (s->search_str, len);
-				for (; fb->pos >= len; --fb->pos)
-				{
-					gchar* current_key = g_utf8_collate_key (current, len);
-					if (g_str_equal (current_key, search_key) &&
-						extra_match (fb, current, current + strlen (s->search_str),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_key);
-						break;
-					}
-					else
-						current = g_utf8_prev_char (current);
-				}
-				g_free (search_key);
+				if (extra_match (fb->pos == 0 && p == search_buf, p, p + keylen, s))
+					break;
+				++p;
 			}
 		}
+	}
+	g_free(search_key);
+	
+	if (p)
+	{
+		/* We've found a match in the normalized buffer.  Now find the
+		 * corresponding position in the source buffer. */
+		gchar* match = normal_advance(
+								direction == SD_BACKWARD ? fb->buf : current,
+								p - search_buf, s->match_case);
+		gchar* match_end = normal_advance(match, keylen, s->match_case);
+		mi = match_info(fb, match, match_end, direction);
+	}
+	
+	g_free(search_buf);
+	return mi;
+}
+
+static gboolean str_is_ascii(gchar* s)
+{
+	for (; *s; ++s)
+		if (!isascii(*s))
+			return FALSE;
+	return TRUE;
+}
+
+/* Returns the next match in the passed buffer.  If direction is SD_BACKWARD,
+   finds the previous match whose end falls before the current position.
+   The search expression should be pre-compiled.  The returned pointer should
+   be freed with match_info_free() when no longer required. */
+MatchInfo *
+get_next_match(FileBuffer *fb, SearchDirection direction, SearchExpression *s)
+{
+	g_return_val_if_fail(fb && s, NULL);
+	
+	if (s->regex)
+		return get_next_regex_match(fb, direction, s);
+	
+	return str_is_ascii(s->search_str) ?
+		get_next_ascii_match(fb, direction, s) :
+		get_next_utf8_match(fb, direction, s);
+}
+
+static gint search_entry_compare(gconstpointer a, gconstpointer b)
+{
+	gchar* a_uri = ((SearchEntry *) a)->uri;
+	gchar* b_uri = ((SearchEntry *) b)->uri;
+	return g_strcmp0(a_uri, b_uri);
+}
+
+static void search_entry_free (gpointer data, gpointer user_data)
+{
+	g_free (((SearchEntry *)data)->uri);
+	g_free (data);
+}
+
+/* Remove duplicate entries in the list, the list must be sorted */
+static GList* search_entries_remove_duplicate (GList *entries)
+{
+	GList *node;
+	
+	for (node = g_list_first (entries); (node != NULL) && (node->next != NULL); )
+	{
+		GList *next = g_list_next (node);
+		SearchEntry *se = (SearchEntry *)node->data;
+		SearchEntry *next_se = (SearchEntry *)next->data;
+		
+		if ((se->te == next_se->te) && (search_entry_compare (se, next_se) == 0))
+		{
+			/* Find a duplicate entry , remove it*/
+			
+			search_entry_free (next_se, NULL);
+			entries = g_list_delete_link (entries, next);
+		}
 		else
 		{
-			/* Forward match */
-			if (s->ignore_case)
-			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gchar* search_caseless = g_utf8_casefold (s->search_str, len);
-				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
-				for (; fb->pos < buf_len; ++fb->pos)
-				{
-					gchar* current_caseless = g_utf8_casefold (current, len);
-					if (g_utf8_collate (current_caseless, search_caseless) == 0 &&
-						extra_match (fb, current, current + strlen (search_caseless),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_caseless);
-						break;
-					}
-					else
-						current = g_utf8_next_char (current);
-				}
-				g_free (search_caseless);
-			}
-			else
-			{
-				gchar* current = g_utf8_offset_to_pointer (fb->buf, fb->pos);
-				gint len = g_utf8_strlen (s->search_str, -1);
-				gint buf_len = g_utf8_strlen (fb->buf, fb->len);
-				gchar* search_key = g_utf8_collate_key (s->search_str, len);
-				for (; fb->pos < buf_len; ++fb->pos)
-				{
-					gchar* current_key = g_utf8_collate_key (current, len);
-					if (g_str_equal (current_key, search_key) &&
-						extra_match (fb, current, current + strlen (s->search_str),
-									 s))
-					{
-						found = TRUE;
-						g_free (current_key);
-						break;
-					}
-					else
-					{
-						g_free (current_key);
-						current = g_utf8_next_char (current);
-					}
-				}
-				g_free (search_key);
-			}
-		}
-		if (found)
-		{
-			mi = g_new0 (MatchInfo, 1);	//better to abort than silently fail to report match ?
-			mi->pos = fb->pos;
-			mi->len = match_len;
-			mi->line = file_buffer_line_from_pos (fb, fb->pos);
-			
-			if (direction == SD_BACKWARD)
-				fb->pos -= match_len;
-			else
-				fb->pos += match_len;
+			/* Get next entry */
+			node = g_list_next(node);
 		}
 	}
-	return mi;
+	
+	return entries;
 }
 
 /* Create list of search entries */
@@ -751,7 +754,7 @@ create_search_entries (Search *s)
 					start = ianjuta_editor_get_position_from_offset (se->te, selstart, NULL);
 					end = ianjuta_editor_get_position_from_offset (se->te, selend, NULL);
 					ianjuta_editor_selection_set(IANJUTA_EDITOR_SELECTION (se->te), 
-				                                 start, end, NULL);	
+				                                 start, end, TRUE, NULL);	
 					g_object_unref (start);
 					g_object_unref (end);
 				}
@@ -763,16 +766,29 @@ create_search_entries (Search *s)
 			{
 				if (IANJUTA_IS_EDITOR (tmp->data))
 				{
+					gchar *uri = NULL;
+
+					if (IANJUTA_IS_FILE (tmp->data))
+					{
+						GFile *file = ianjuta_file_get_file (IANJUTA_FILE (tmp->data), NULL);
+						
+						if (file != NULL)
+						{
+							uri = g_file_get_uri (file);
+							g_object_unref (file);
+						}
+					}
+					
 					se = g_new0 (SearchEntry, 1);
-				se->type = SE_BUFFER;
+					se->type = SE_BUFFER;
 					se->te = IANJUTA_EDITOR (tmp->data);
-				se->direction = SD_FORWARD;
-				se->start_pos = 0;
-				se->end_pos = -1;
-				entries = g_list_prepend(entries, se);
+					se->uri = uri;
+					se->direction = SD_FORWARD;
+					se->start_pos = 0;
+					se->end_pos = -1;
+					entries = g_list_prepend(entries, se);
+				}
 			}
-			}
-			entries = g_list_reverse(entries);
 			g_list_free (editors);
 			break;
 		case SR_FILES:
@@ -785,9 +801,9 @@ create_search_entries (Search *s)
 			anjuta_shell_get (ANJUTA_PLUGIN(sr->docman)->shell,
 							  "project_root_uri", G_TYPE_STRING,
 							  &dir_uri, NULL);
-			// FIXME : Replace Standard UNIX IO functions by gnome-vfs 
+			// FIXME : Replace Standard UNIX IO functions by gio 
 			if (dir_uri)			
-				dir = gnome_vfs_get_local_path_from_uri(dir_uri);
+				dir = anjuta_util_get_local_path_from_uri(dir_uri);
 			
 			if (!dir)
 			{
@@ -796,10 +812,7 @@ create_search_entries (Search *s)
 				dir = g_get_current_dir();
 			}
 
-			if (SR_FILES == s->range.type)
-				files = create_search_files_list(&(s->range.files), dir);
-			else /* if (SR_PROJECT == s->range.type) */
-				files = get_project_file_list();	
+			files = get_project_file_list();	
 			
 			if (files)
 			{
@@ -807,7 +820,7 @@ create_search_entries (Search *s)
 				{
 					se = g_new0(SearchEntry, 1);
 					se->type = SE_FILE;
-					se->path = (char *) tmp->data;
+					se->uri = (char *) tmp->data;
 					se->direction = SD_FORWARD;
 					se->type = SE_FILE;
 					se->start_pos = 0;
@@ -815,16 +828,25 @@ create_search_entries (Search *s)
 					entries = g_list_prepend(entries, se);
 				}
 				g_list_free(files);
-				entries = g_list_reverse(entries);
 			}
 			g_free(dir);
 			g_free(dir_uri);
 			break;
 		}
-	}	
+	}
+
+	entries = g_list_sort(entries, search_entry_compare);
+	entries = search_entries_remove_duplicate (entries);
+	
 	return entries;		
 }
 
+void free_search_entries (GList *entries)
+{
+	g_list_foreach (entries, search_entry_free, NULL);
+	g_list_free (entries);
+}
+
 gchar *
 regex_backref (MatchInfo *mi, FileBuffer *fb)
 {
